<?php

class BifitKassa
{
	const MODULE_ID = 'bifit.kassa';

	public function __construct()
	{
	}

	public function __saveLog($data, $order_number)
	{
		$file = dirname(__FILE__) . '/logs/log_' . $order_number . '.txt';
		$top =
			"## " .
			date('Y-m-d H:i:s') .
			" | " .
			GetMessage('BIFIT_LOG_BIFIT_KASSA') .
			" | " .
			GetMessage('BIFIT_LOG_ORDER_NUMBER') .
			$order_number;
		$len = mb_strlen($top, "UTF-8");
		for ($i = 0; $i < $len; $i++) {
			$line .= "#";
		}
		$current = file_get_contents($file);
		$content =
			$current . $line . "\n" . $top . "\n" . $line . "\n" . $data . "\n";
		file_put_contents($file, $content);
	}

	protected function getPayment($orderId)
	{
		$dbRes = \Bitrix\Sale\PaymentCollection::getList(array(
			'select' => array('*'),
			'filter' => array(
				'=ORDER_ID' => $orderId
			)
		));

		$sumCash = 0;
		$sumEcash = 0;

		while ($item = $dbRes->fetch()) {
			$paySystemId = $item['PAY_SYSTEM_ID'];
			$arFilter = array(
				'PAY_SYSTEM_ID' => $paySystemId
			);
			$queryPaymentSystem = CSalePaySystemAction::GetList(
				array(),
				$arFilter,
				false,
				false,
				['*']
			);
			$paymentSystem = $queryPaymentSystem->Fetch();

			if ($paymentSystem['IS_CASH'] == 'Y') {
				$sumCash += $item['SUM'];
			} else {
				$sumEcash += $item['SUM'];
			}
		}

		return array(
			'cash' => $sumCash,
			'ecash' => $sumEcash
		);
	}

	function __formatprc($price)
	{
		CModule::IncludeModule("sale");

		$decimals = 2;
		$formatInfo = CCurrencyLang::GetCurrencyFormat(
			COption::GetOptionString("sale", "default_currency", "RUB")
		);

		if (isset($formatInfo['DECIMALS'])) {
			$decimals = intval($formatInfo['DECIMALS']);
		}

		$dec_count = $decimals;

		$price =
			$dec_count < 0
				? round($price, $dec_count)
				: number_format($price, $dec_count, '.', '');

		return $price;
	}

	function __checkInn($inn, $type = 1)
	{
		// check the fullness
		if (isset($inn) && trim($inn) != '') {
			if ($type == 2) {
				// only numbers in the amount of 10 pcs.
				$inn = !preg_match("/^\d{10}$/", trim($inn))
					? GetMessage('BIFIT_WRONG_INN')
					: trim($inn);
			} elseif ($type == 1) {
				// only numbers in the amount of 12 pcs.
				$inn = !preg_match("/^\d{12}$/", trim($inn))
					? GetMessage('BIFIT_WRONG_INN')
					: trim($inn);
			} else {
				// only numbers in the amount of 10 or 12 pieces.
				$inn = !preg_match("/^\d{10}$|^\d{12}$/", trim($inn))
					? GetMessage('BIFIT_WRONG_INN')
					: trim($inn);
			}
		} else {
			// TIN is empty or not found!
			$inn = GetMessage('BIFIT_EMPTY_INN');
		}

		return $inn;
	}

	function __checkPhone($tel)
	{
		// check the fullness
		if (isset($tel) && trim($tel) != '') {
			// remove extra characters
			$phone = str_replace(
				array('+', '(', ')', '-', ' '),
				'',
				trim($tel)
			);

			// get the first digit
			$first = substr($phone, "0", 1);

			// if the number is 10 digits and the first digit is not 7, then add 7
			$phone =
				strlen($phone) == 10 && $first != 7 ? '7' . $phone : $phone;

			// Final inspection
			$phone = !preg_match("/^[0-9]{11}+$/", $phone)
				? str_replace(
					GetMessage('BIFIT_PHONE'),
					GetMessage('BIFIT_PHONE') . ' ' . $phone,
					GetMessage('BIFIT_WRONG_PHONE')
				)
				: $phone;
		} else {
			// The phone is empty or not found!
			$phone = GetMessage('BIFIT_EMPTY_PHONE');
		}

		return $phone;
	}

	function __checkEmail($email)
	{
		// check fullness
		if (isset($email) && trim($email) != '') {
			// record model check
			$email = !preg_match("/^\+\d+$|^\S+@\S+$/", $email)
				? str_replace(
					GetMessage('BIFIT_EMAIL'),
					GetMessage('BIFIT_EMAIL') . ' ' . $email,
					GetMessage('BIFIT_WRONG_EMAIL')
				)
				: $email;
		} else {
			// Email is empty or not found!
			$email = GetMessage('BIFIT_EMPTY_EMAIL');
		}

		return $email;
	}

	function __checkProductPrice(
		$item_price,
		$order_subtotal,
		$order_discount = 0
	) {
		// if the discount is more than zero
		if ($order_discount > 0) {
			// calculate the discount percentage
			$percent = ($order_discount * 100) / $order_subtotal;

			// reduce the price of goods according to the discount
			$item_price = $item_price - $item_price * ($percent / 100);
		}

		return $item_price;
	}

	function __checkResponce($response)
	{
		// error conditions
		if (
			is_object($response) &&
			($response->type == 'ERROR' || $response->status == 400)
		) {
			$mess = '';
			if ($response->type == 'ERROR') {
				$array = json_decode(json_encode($response), true);
				array_walk_recursive($array, function ($item, $key) use (
					&$mess
				) {
					if ($item != 'ERROR' && trim($item != '')) {
						$mess .= $item . "\n";
					}
				});
			} elseif ($response->status == 400) {
				$mess .= $response->error . "\n";
				$mess .= $response->message . "\n";
			}

			return $mess;
		}

		return false;
	}

	function __loadData($url, $headers = false, $data = false, $type = 'POST')
	{
		// initialize the cURL session
		$ch = curl_init($url);
		if ($type != 'POST') {
			curl_setopt($ch, CURLOPT_CUSTOMREQUEST, $type);
		} else {
			curl_setopt($ch, CURLOPT_POST, true);
		}

		// add data
		if ($data) {
			curl_setopt($ch, CURLOPT_POSTFIELDS, $data);
		}

		// returning the result as a string
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
		curl_setopt($ch, CURLOPT_ENCODING, '');
		curl_setopt($ch, CURLOPT_MAXREDIRS, 10);
		curl_setopt($ch, CURLOPT_TIMEOUT, 0);
		curl_setopt($ch, CURLOPT_FOLLOWLOCATION, true);
		curl_setopt($ch, CURLOPT_HTTP_VERSION, CURL_HTTP_VERSION_1_1);

		// add headers
		if ($headers) {
			curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);
		}

		// get the answer
		$output = curl_exec($ch);

		// close the cURL session
		curl_close($ch);

		return $output;
	}

	function __getToken($login, $password)
	{
		$signature = base64_encode(hash('sha256', $password, true));
		$data = 'username=' . $login;
		$data .= '&password=' . $signature;
		$data .= '&client_id=cashdesk-rest-client';
		$data .= '&client_secret=cashdesk-rest-client';
		$data .= '&grant_type=password';
		$url = 'https://kassa.bifit.com/cashdesk-api/v1/oauth/token';
		$headers = array('Content-Type: application/x-www-form-urlencoded');
		$tokenData = json_decode(self::__loadData($url, $headers, $data));

		return $tokenData;
	}

	function __getOrganizations($token)
	{
		$url =
			'https://kassa.bifit.com/cashdesk-api/v1/protected/organizations/list/read';
		$headers = array('Authorization: Bearer ' . $token);
		$orgData = json_decode(self::__loadData($url, $headers));

		return $orgData;
	}

	function __getTradeObjects($org_id, $token)
	{
		$data = 'organization_id=' . $org_id;
		$url =
			'https://kassa.bifit.com/cashdesk-api/v1/protected/trade_objects/list/read';
		$headers = array(
			'Content-Type: application/x-www-form-urlencoded',
			'Authorization: Bearer ' . $token
		);
		$tradeData = json_decode(self::__loadData($url, $headers, $data));

		return $tradeData;
	}

	function __createMainArray($orgData, $token)
	{
		$arr = [];

		foreach ($orgData as $key => $org) {
			$arr[$key] = new stdClass();
			$arr[$key]->id = $org->id;
			$arr[$key]->name = $org->name;
			$arr[$key]->traides = [];

			$tradeData = self::__getTradeObjects($org->id, $token);

			foreach ($tradeData as $k => $trd) {
				$arr[$key]->traides[$k] = new stdClass();
				$arr[$key]->traides[$k]->id = $trd->id;
				$arr[$key]->traides[$k]->name = $trd->name;
			}
		}

		return $arr;
	}

	function __createSelects($array, $name, $org_id = null)
	{
		$html = '<select name="' . $name . '">';

		foreach ($array as $key => $org) {
			if ($name == 'ORGANIZATION') {
				$html .= '<option value="' . $org->id . '">';
				$html .= $org->name;
				$html .= '</option>';
			}

			if ($name == 'TRADE_OBJECT' && $org->id != $org_id) {
				continue;
			}

			if ($name != 'ORGANIZATION') {
				foreach ($org->traides as $k => $trd) {
					$html .= '<option value="' . $trd->id . '">';
					$html .= $trd->name;
					$html .= '</option>';
				}
			}
		}

		$html .= '</select>';

		return $html;
	}

	function __saveReport($test, $bifit, $receipt, $receipt_id, $doc, $object)
	{
		// display the property of the operation
		$log =
			$test != ''
				? '####### ' . GetMessage('BIFIT_TEST_REQUEST') . "\n"
				: '####### ' . GetMessage('BIFIT_BATTLE_REQUEST') . "\n";

		// set the time zone according to the site settings
		$log .= '####### ' . date('Y-m-d H:i:s', time()) . "\n";

		// ANSWER #1

		$log .= '####### ' . GetMessage('BIFIT_ANSWER') . ' #1: ' . "\n";

		// response data # 1
		if (is_object($bifit)) {
			foreach ($bifit as $key => $val) {
				$log .= $key . ': ' . $val . "\n";
			}
		}

		// RECEIPT

		// generated receipt for sending
		$log .=
			'####### ' .
			GetMessage('BIFIT_CHECK') .
			': ' .
			"\n" .
			$receipt .
			"\n";

		// ANSWER #2

		$log .= '####### ' . GetMessage('BIFIT_ANSWER') . ' #2: ' . "\n";

		// response data #2
		if (is_object($receipt_id)) {
			$array = json_decode(json_encode($receipt_id), true);
			array_walk_recursive($array, function ($item, $key) use (&$log) {
				if ($item != 'ERROR' && trim($item) != '') {
					$log .= $item . "\n";
				}
			});
		} else {
			$log .= $receipt_id . "\n";

			// ANSWER #3

			$log .= '####### ' . GetMessage('BIFIT_ANSWER') . ' #3: ' . "\n";
			$log .= is_object($doc) ? print_r($bifit, true) : '';

			// LINK TO THE CHECK

			// generated link to the check
			$log .=
				'####### ' .
				GetMessage('BIFIT_LINK_TO_CHECK') .
				': ' .
				"\n" .
				$object->receipt_link;
		}

		$object->log = $log;

		// SAVE IN DB

		global $DB;

		$connection = Bitrix\Main\Application::getConnection();
		$sqlHelper = $connection->getSqlHelper();
		$connection = \Bitrix\Main\Application::getConnection();
		$connection->queryExecute(
			"INSERT INTO b_bifit_kassa_reports (
			ORDER_ID, 
			ORDER_NUMBER,
			ORDER_DATE,
			ACTION_DATE,
			RECEIPT_LINK,
			RECEIPT_ID,
			RECEIPT_TYPE,
			LOG
			) VALUES ('" .
				$sqlHelper->forSql($object->order_id) .
				"', 
			'" .
				$sqlHelper->forSql($object->order_number) .
				"', 
			'" .
				$sqlHelper->forSql($object->order_date) .
				"', 
			'" .
				$sqlHelper->forSql($object->action_date) .
				"', 
			'" .
				$sqlHelper->forSql($object->receipt_link) .
				"', 
			'" .
				$sqlHelper->forSql($object->receipt_id) .
				"', 
			'" .
				$sqlHelper->forSql($object->receipt_type) .
				"',
			'" .
				$sqlHelper->forSql($object->log) .
				"'
			)"
		);
	}

	function __saveOrder($lk)
	{
		Bitrix\Main\Localization\Loc::loadMessages(__FILE__);

		global $DB;

		// Options
		$moduleId = self::MODULE_ID;
		$login = self::__checkPhone(
			COption::GetOptionString($moduleId, 'login')
		);
		$password = COption::GetOptionString($moduleId, 'password');
		$organization_id = COption::GetOptionString($moduleId, 'organization');
		$trade_object_id = COption::GetOptionString($moduleId, 'trade_object');
		$use_refused = COption::GetOptionString($moduleId, 'lk_use_refused')
			? 1
			: null;

		// log
		$logData = '## ' . GetMessage('BIFIT_LOG_GETTING_A_TOKEN') . "\n";

		// Get token
		$tokenData = self::__getToken($login, $password);
		$token = $tokenData->access_token;

		// log
		if ($token) {
			$logData .=
				'## ' .
				GetMessage('BIFIT_LOG_GOT_A_TOKEN') .
				': ' .
				$token .
				"\n";
		} else {
			$logData .= '## ' . GetMessage('BIFIT_LOG_TOKEN_ERROR') . "\n";
		}

		// parameters for ordering
		$onlineOrderItems = '';

		// log
		$logData .=
			'## ' . GetMessage('BIFIT_LOG_CHECK_AVAILABILITY_PRODUCT') . "\n";

		// form order data
		foreach ($lk->r_products as $key_id => $product) {
			// address for POST request
			$url =
				'https://kassa.bifit.com/cashdesk-api/v1/protected/nomenclatures/external_id/' .
				$product->product_id .
				'?organization_id=' .
				$organization_id;

			// link data
			$data = false;

			// headers
			$headers = array(
				'Content-Type: application/x-www-form-urlencoded',
				'Authorization: Bearer ' . $token
			);

			$prod = json_decode(self::__loadData($url, $headers, $data, 'GET'));

			if ($prod == null) {
				// CREATE A NEW PRODUCT

				// log
				$logData .=
					'## ' .
					GetMessage('BIFIT_LOG_PRODUCT_VS_ID') .
					$product->product_id .
					' ' .
					GetMessage('BIFIT_LOG_NOT_FOUND_IN_BIFIT_DATABASE') .
					"\n";
				$logData .=
					'## ' . GetMessage('BIFIT_LOG_ADD_NEW_PRODUCT') . "\n";

				// address for POST request
				$url =
					'https://kassa.bifit.com/cashdesk-api/v1/protected/nomenclatures';

				// link data
				$data = '{';
				$data .= '"item": {';
				$data .=
					'"vendorCode": "' . $product->product_vendor_code . '", ';
				$data .= '"name":"' . $product->product_name . '", ';
				$data .= '"externalId":"' . $product->product_id . '", ';
				$data .= '"sellingPrice":' . $product->product_item_price;
				$data .= '},';
				$data .= '"organizationId":"' . $organization_id . '"';
				$data .= '}';

				// headers
				$headers = array(
					'Content-Type: application/json',
					'Authorization: Bearer ' . $token
				);

				// log
				$logData .=
					GetMessage('BIFIT_LOG_DATA_FOR_REQUEST') . ':' . "\n";
				$logData .= $data . "\n";

				// get data from request
				$tov = json_decode(self::__loadData($url, $headers, $data));

				// log
				if (!is_object($tov) && !is_array($tov)) {
					$logData .=
						'## ' .
						GetMessage('BIFIT_LOG_SUCC_ADD_NEW_PRODUCT') .
						$tov .
						"\n";
				} else {
					$logData .=
						'## ' .
						GetMessage('BIFIT_LOG_ERROR_WHEN_ADDED_NEW_PRODUCT') .
						':';
					$logData .= self::__checkResponce($tov) . "\n";
				}
			} else {
				$tov = $prod->id;

				// log
				$logData .=
					'## ' .
					GetMessage('BIFIT_LOG_PRODUCT_VS_ID') .
					$product->product_id .
					' ' .
					GetMessage('BIFIT_LOG_FOUND_IN_BIFIT_DATABASE_VS_ID') .
					$tov .
					"\n";
			}

			// display product data in the order
			$onlineOrderItems .= $onlineOrderItems != '' ? ', ' : '';
			$onlineOrderItems .= '{';
			$onlineOrderItems .= '"nomenclatureId":"' . $tov . '", ';
			$onlineOrderItems .= '"paymentSubject":"PRODUCT", ';
			$onlineOrderItems .= '"name":"' . $product->product_name . '", ';
			$onlineOrderItems .=
				'"price":' .
				self::__formatprc($product->product_item_price) .
				', ';
			$onlineOrderItems .= '"quantity":' . $product->product_quantity;
			$onlineOrderItems .= '}';
		}

		// delivery details
		if ($lk->order_price_delivery > 0) {
			// address for POST request
			$url =
				'https://kassa.bifit.com/cashdesk-api/v1/protected/nomenclatures/external_id/' .
				'delivery_' .
				$lk->order_delivery_id .
				'?organization_id=' .
				$organization_id;

			// link data
			$data = false;

			// headers
			$headers = array(
				'Content-Type: application/x-www-form-urlencoded',
				'Authorization: Bearer ' . $token
			);

			$delivery = json_decode(
				self::__loadData($url, $headers, $data, 'GET')
			);

			// log
			$logData .=
				'## ' . GetMessage('BIFIT_LOG_CHECK_DELIVERY_SERVICE') . "\n";

			if ($delivery == null) {
				// CREATE A NEW SERVICE

				// log
				$logData .=
					'## ' .
					GetMessage('BIFIT_LOG_DELIVERY_SERVICE_NOT_FOUND') .
					"\n";
				$logData .=
					'## ' .
					GetMessage('BIFIT_LOG_ADD_NEW_DELIVERY_SERVICE') .
					"\n";

				// address for POST request
				$url =
					'https://kassa.bifit.com/cashdesk-api/v1/protected/nomenclatures';

				// link data
				$data = '{';
				$data .= '"item": {';
				$data .= '"name":"' . GetMessage('BIFIT_DELIVERY') . '", ';
				$data .=
					'"externalId":"' .
					'delivery_' .
					$lk->order_delivery_id .
					'", ';
				$data .= '"sellingPrice":' . $lk->order_price_delivery;
				$data .= '},';
				$data .= '"organizationId":"' . $organization_id . '"';
				$data .= '}';

				// headers
				$headers = array(
					'Content-Type: application/json',
					'Authorization: Bearer ' . $token
				);

				// log
				$logData .=
					GetMessage('BIFIT_LOG_DATA_FOR_REQUEST') . ':' . "\n";
				$logData .= $data . "\n";

				// get data from request
				$tov = json_decode(self::__loadData($url, $headers, $data));

				// log
				if (!is_object($tov) && !is_array($tov)) {
					$logData .=
						'## ' .
						GetMessage('BIFIT_LOG_SUCC_ADD_NEW_DELIVERY_SERVICE') .
						$tov .
						"\n";
				} else {
					$logData .=
						'## ' .
						GetMessage(
							'BIFIT_LOG_ERROR_WHEN_ADDED_NEW_DELIVERY_SERVICE'
						) .
						"\n";
					$logData .= self::__checkResponce($tov) . "\n";
				}
			} else {
				$tov = $delivery->id;

				// log
				$logData .=
					'## ' .
					GetMessage(
						'BIFIT_LOG_DELIVERY_SERVICE_ALREADY_EXIST_VS_ID'
					) .
					$tov .
					"\n";
			}

			// display data of delivery method in order
			$onlineOrderItems .= $onlineOrderItems != '' ? ', ' : '';
			$onlineOrderItems .= '{';
			$onlineOrderItems .= '"nomenclatureId":"' . $tov . '", ';
			$onlineOrderItems .= '"paymentSubject":"SERVICE", ';
			$onlineOrderItems .=
				'"name":"' . GetMessage('BIFIT_DELIVERY') . '", ';
			$onlineOrderItems .=
				'"price":' .
				self::__formatprc($lk->order_price_delivery) .
				', ';
			$onlineOrderItems .= '"quantity":1';
			$onlineOrderItems .= '}';
		}

		// formatting time
		//$time = intval(microtime(true) * 1000);
		$objDateTime = new \Bitrix\Main\Type\DateTime();
		$timestamp = $objDateTime->getTimestamp();
		//$time = strlen($timestamp) == 10 ? $timestamp * 1000 : $timestamp;
		$time = $timestamp;

		// CREATE A USER IN THE CASH

		// address for POST request
		$url = 'https://kassa.bifit.com/cashdesk-api/v1/protected/clients';

		// headers
		$headers = array(
			'Content-Type: application/json',
			'Authorization: Bearer ' . $token
		);

		// link data
		$data = '{';
		$data .= '"organizationId":"' . $organization_id . '", ';
		$data .= '"address":"' . $lk->address . '", ';
		$data .= '"phone":"' . $lk->user_phone . '", ';
		$data .= '"lastName":"' . trim($lk->user_last_name) . '", ';
		$data .= '"firstName":"' . trim($lk->user_name) . '", ';
		$data .= '"email":"' . $lk->user_email . '"';
		$data .= '}';

		// log
		$logData .= '## ' . GetMessage('BIFIT_LOG_CHECK_CLIENT') . "\n";
		$logData .=
			'## ' . GetMessage('BIFIT_LOG_DATA_FOR_REQUEST') . ':' . "\n";
		$logData .= $data . "\n";

		// trying to add a new client
		$clientId = json_decode(self::__loadData($url, $headers, $data));

		// such a client is already in the database
		if ($clientId->type == "ERROR") {
			// address for POST request
			$url =
				'https://kassa.bifit.com/cashdesk-api/v1/protected/clients/list/read?organization_id=' .
				$organization_id;

			// link data
			$data = false;

			// headers
			$headers = array(
				'Content-Type: application/json',
				'Authorization: Bearer ' . $token
			);

			// get all clients
			$clients = json_decode(self::__loadData($url, $headers, $data));

			foreach ($clients as $clientsItem) {
				// phone number match
				if ($clientsItem->phone == $lk->user_phone) {
					$clientId = $clientsItem->id;
				}
			}

			// log
			$logData .=
				'## ' .
				GetMessage('BIFIT_LOG_CLIENT_ALREADY_EXIST') .
				$clientId .
				"\n";
		} else {
			// log
			$logData .=
				'## ' .
				GetMessage('BIFIT_LOG_ADDED_NEW_CLIENT_VS_ID') .
				$clientId .
				"\n";
		}

		// SENDING AN ORDER TO THE CASHIER

		// checking order availability
		// address for GET request
		$url =
			'https://kassa.bifit.com/cashdesk-api/v1/protected/online_orders/list/read/' .
			$clientId .
			'?organization_id=' .
			$organization_id;

		// link data
		$data = false;

		// headers
		$headers = array(
			'Content-Type: application/x-www-form-urlencoded',
			'Authorization: Bearer ' . $token
		);

		// get orders
		$clientOrders = json_decode(
			self::__loadData($url, $headers, $data, 'GET')
		);

		$receipt_id = 0;
		$del_type = '';

		// log
		$logData .= '## ' . GetMessage('BIFIT_LOG_CHECK_CURR_ORDER') . "\n";

		// search order by id
		foreach ($clientOrders as $key => $clientOrder) {
			if ($clientOrder->externalId == $lk->order_number) {
				$receipt_id = $clientOrder->id;
				$del_type = $clientOrder->deliveryType;

				// log
				$logData .=
					'## ' .
					GetMessage('BIFIT_LOG_THIS_ORDER_ALREADY_EXIST') .
					$receipt_id .
					'. ' .
					GetMessage('BIFIT_LOG_DELIVERY_TYPE') .
					$del_type .
					'.' .
					"\n";

				break;
			}
		}

		// log
		if (!$receipt_id) {
			$logData .=
				'## ' . GetMessage('BIFIT_LOG_CURR_ORDER_NOT_FOUND') . "\n";
		}

		// type of delivery
		$lk_courier_deliveries = json_decode(
			COption::GetOptionString($moduleId, "lk_courier_delivery")
		);
		$lk_self_deliveries = json_decode(
			COption::GetOptionString($moduleId, "lk_self_delivery")
		);

		$deliveryType = 'SERVICE';
		if (
			isset($lk_courier_deliveries) &&
			in_array($lk->order_delivery_id, $lk_courier_deliveries)
		) {
			$deliveryType = 'COURIER';
		} elseif (
			isset($lk_self_deliveries) &&
			in_array($lk->order_delivery_id, $lk_self_deliveries)
		) {
			$deliveryType = 'SELF';
		}

		// assign "is_update"
		$lk_refused_statuses = json_decode(
			COption::GetOptionString($moduleId, "lk_refused_statuses")
		);
		$is_update =
			$receipt_id > 0 &&
			isset($lk_refused_statuses) &&
			in_array($lk->order_status_id, $lk_refused_statuses) &&
			$use_refused
				? 1
				: null;

		// new order || "is_update" conditions
		if ($deliveryType != 'SERVICE' && ($receipt_id == 0 || $is_update)) {
			// address for request
			$url =
				'https://kassa.bifit.com/cashdesk-api/v1/protected/online_orders';

			// headers
			$headers = array(
				'Content-Type: application/json',
				'Authorization: Bearer ' . $token
			);

			// data
			if ($is_update) {
				// log
				$logData .=
					'## ' . GetMessage('BIFIT_LOG_START_ORDER_UPDATE') . "\n";

				$url .= '/' . $receipt_id . '/status';
				$type = 'PUT';
				$statusType = $del_type == 'SELF' ? 'REFUSED' : 'DELETED';
				// Basic data
				$data = '{';
				$data .= '"onlineOrderId": ' . $lk->order_number . ',';
				$data .= '"statusType": "' . $statusType . '",';
				$data .= '"statusDate": 0,';
				$data .= '"statusInfo": ""';
				$data .= '}';
			} else {
				// log
				$logData .=
					'## ' . GetMessage('BIFIT_LOG_START_NEW_ORDER') . "\n";

				// data
				$type = 'POST';
				$statusType =
					$deliveryType == 'SELF'
						? 'ACCEPTED'
						: COption::GetOptionString(
							$moduleId,
							'lk_new_courier_order_status',
							'NEW'
						);
				// Basic data
				$onlineOrder = '{';
				$onlineOrder .= '"organizationId":"' . $organization_id . '", ';
				$onlineOrder .= '"tradeObjectId":"' . $trade_object_id . '", ';
				$onlineOrder .= '"taxSystem":"' . $lk->tax_system . '", ';
				$onlineOrder .= '"externalId":"' . $lk->order_number . '", ';
				$onlineOrder .= '"deliveryType":"' . $deliveryType . '", ';
				$onlineOrder .= '"deliveryAmount":"' . $deliveryAmount . '", ';
				$onlineOrder .= '"paid":false, ';
				$onlineOrder .= '"orderTime":' . $time . ', ';
				$onlineOrder .= '"clientId":' . $clientId . ', ';
				$onlineOrder .= '"address":"' . $lk->user_address . '", ';
				$onlineOrder .=
					'"comment":"' .
					htmlspecialchars($lk->order_comments) .
					'", ';
				$onlineOrder .= '"responsiblePersonLogin":"' . $login . '", ';
				$onlineOrder .=
					'"totalAmount":' .
					self::__formatprc($lk->order_price) .
					', ';
				$onlineOrder .= '"currentStatusType":"' . $statusType . '", ';
				$onlineOrder .= '"currentStatusTime":0';
				$onlineOrder .= '}';

				$data = '{';
				$data .= '"onlineOrder":' . $onlineOrder . ', ';
				$data .= '"onlineOrderItems":[' . $onlineOrderItems . ']';
				$data .= '}';
			}

			// log
			$logData .=
				'## ' .
				GetMessage('BIFIT_LOG_ORDER_DATA_FOR_REQUEST') .
				':' .
				"\n";
			$logData .= $data . "\n";

			// get the ID of the document being processed (id)
			$receipt_id = json_decode(
				self::__loadData($url, $headers, $data, $type)
			);

			// log
			if (!is_object($receipt_id) && !is_array($receipt_id)) {
				if ($is_update && $statusType == 'DELETED') {
					$logData .=
						'## ' .
						GetMessage('BIFIT_LOG_SUCC_ORDER_DEL') .
						$receipt_id .
						"\n";
				} else {
					$logData .=
						'## ' .
						GetMessage('BIFIT_LOG_SUCC_ORDER_SAVE') .
						$receipt_id .
						"\n";
				}
			} else {
				$logData .=
					'## ' .
					GetMessage('BIFIT_LOG_ERR_IN_SAVE_ORDER_PROC') .
					':' .
					"\n";
				$logData .= self::__checkResponce($receipt_id) . "\n";
			}
		}

		// send payment information
		if (
			$lk->order_payed == 'Y' &&
			in_array($lk->calc_method, [
				'PREPAY_FULL',
				'PREPAY_PARTIAL',
				'FULL_PAY'
			])
		) {
			// log
			$logData .=
				'## ' . GetMessage('BIFIT_LOG_START_PAY_DATA_PROC') . "\n";

			// address for request
			$url =
				'https://kassa.bifit.com/cashdesk-api/v1/protected/online_orders/payments';

			// headers
			$headers = array(
				'Content-Type: application/json',
				'Authorization: Bearer ' . $token
			);

			// data
			$type = 'POST';
			$data = '{';
			$data .= '"onlineOrderExternalId": ' . $lk->order_number . ',';
			$data .= '"organizationId":"' . $organization_id . '", ';
			$data .= '"calculationMethod":"' . $lk->calc_method . '", ';
			$data .= '"total": ' . $lk->order_price;
			$data .= '}';

			// log
			$logData .=
				'## ' .
				GetMessage('BIFIT_LOG_PAY_DATA_FOR_REQUEST') .
				':' .
				"\n";
			$logData .= $data . "\n";

			// get the ID of the document being processed (id)
			$receipt_id = json_decode(
				self::__loadData($url, $headers, $data, $type)
			);

			// log
			if (!is_object($receipt_id) && !is_array($receipt_id)) {
				$logData .=
					'## ' .
					GetMessage('BIFIT_LOG_SUCC_SEND_PAY_DATA') .
					$receipt_id .
					"\n";
			} else {
				$logData .=
					'## ' .
					GetMessage('BIFIT_LOG_ERR_WHEN_SEND_PAY_DATA') .
					':' .
					"\n";
				$logData .= self::__checkResponce($receipt_id) . "\n";
			}
		}

		// save log
		$logging = COption::GetOptionString($moduleId, 'logging', 1);
		if ($logging) {
			self::__saveLog($logData, $lk->order_number);
		}
	}

	function __mainProcess($order, $test, $token, $i)
	{
		Bitrix\Main\Localization\Loc::loadMessages(__FILE__);

		global $DB;

		$moduleId = self::MODULE_ID;

		// 1. AUTHORIZATION
		//    process necessary to connect a client to fiscal processing

		// address for POST request
		$url = 'https://fp' . $test . '.bifit.com/processing-api/oauth/token';

		// link data
		$data = 'token=' . $token;
		$data .= '&client_id=processing-connector-token';
		$data .= '&client_secret=processing-connector-token';
		$data .= '&grant_type=token';

		// headers
		$headers = array('Content-Type: application/x-www-form-urlencoded');

		// decode the received json response
		$bifit = json_decode(self::__loadData($url, $headers, $data));

		// 2. FORMATION OF THE CONTENT OF A CHECK
		//    formation of information for further sending for fiscalization

		// Plugin parameters

		// cashier name
		$cashier_name = COption::GetOptionString(
			$moduleId,
			'cashier_name',
			'Иванов И.И.'
		);
		// TIN of the cashier
		$cashier_inn = COption::GetOptionString(
			$moduleId,
			'cashier_inn',
			'123456789012'
		);
		// document type
		$arr_rec_type = array(
			'1' => 'SALE',
			'2' => 'SALE_RETURN',
			'3' => 'PURCHASE',
			'4' => 'PURCHASE_RETURN'
		);
		$receipt_type = $arr_rec_type[$i];
		// tax system
		$tax_system = COption::GetOptionString(
			$moduleId,
			"tax_system_{$i}",
			'SIMPLIFIED_WITH_EXPENSE'
		);
		// method of calculation
		$calc_method = COption::GetOptionString(
			$moduleId,
			'calculation_method',
			'FULL_PAY'
		);
		// uniform VAT
		$vat_mode = COption::GetOptionString($moduleId, 'vat_mode', '1');
		// VAT
		$vat = COption::GetOptionString($moduleId, 'vat', 'WITHOUT_VAT');
		// payment type
		$payment_type = COption::GetOptionString(
			$moduleId,
			'payment_type',
			'CARD'
		);
		// place of settlement
		$payment_address = COption::GetOptionString(
			$moduleId,
			'payment_address',
			'https://site.ru'
		);
		// client address type
		$address_type = COption::GetOptionString(
			$moduleId,
			'address_type',
			'email'
		);
		// Full name of the client
		$fio = COption::GetOptionString($moduleId, 'fio', '0');

		// buyer
		/// get the buyer's data
		$user = CUSer::GetByID($order['USER_ID'])->Fetch();
		/// buyer's phone
		$user_phone = 0;
		/// buyer's email
		$user_email = isset($user['EMAIL']) ? $user['EMAIL'] : 0;

		// order data
		/// order id
		$order_id = (int) $order['ID'];
		/// order number
		$order_number = $order['ACCOUNT_NUMBER'];

		$dbOrderProps = CSaleOrderPropsValue::GetList(
			["SORT" => "ASC"],
			["ORDER_ID" => $order_id]
		);

		while ($arOrderProps = $dbOrderProps->Fetch()) {
			if (
				$arOrderProps['CODE'] == "EMAIL" &&
				$arOrderProps['VALUE'] != ''
			) {
				$user_email = $arOrderProps['VALUE'];
			}
			if (
				$arOrderProps['CODE'] == "PHONE" &&
				$arOrderProps['VALUE'] != ''
			) {
				$user_phone = $arOrderProps['VALUE'];
			}
		}

		// buyer's address on the receipt (phone or email)
		if ($address_type == 'phone' && $user_phone) {
			$address = self::__checkPhone($user_phone);
		} elseif ($address_type == 'phone' && !$user_phone && $user_email) {
			$address = self::__checkEmail($user_email);
		} elseif ($address_type == 'email' && $user_email) {
			$address = self::__checkEmail($user_email);
		} elseif ($address_type == 'email' && !$user_email && $user_phone) {
			$address = self::__checkPhone($user_phone);
		} else {
			$address =
				$address_type == 'phone' && $user
					? self::__checkPhone($user_phone)
					: self::__checkEmail($user_email);
		}

		// order payment items
		/// cost of delivery
		$o_shipment = $order['PRICE_DELIVERY'];

		// array parameters
		/// an array of products to fill
		$r_products = array();
		/// an array of order items
		$dbBasket = CSaleBasket::GetList(
			["ID" => "ASC"],
			["ORDER_ID" => $order_id],
			false,
			false
		);

		// key of the array of goods for the receipt
		$key_id = 0;

		// iterate over the array of order items
		while ($product = $dbBasket->GetNext()) {
			$res = CIBlockElement::GetByID($product['PRODUCT_ID']);

			if ($ar_res = $res->GetNext()) {
				$iblockid = $ar_res['IBLOCK_ID'];
			}

			$db_props = CIBlockElement::GetProperty(
				$iblockid,
				$product['PRODUCT_ID'],
				["SORT" => "ASC"],
				["CODE" => "ART_CODE"]
			);

			if ($ar_props = $db_props->Fetch()) {
				$productVendorCode = IntVal($ar_props["VALUE"]);
			}

			/// adjusting the price of the product taking into account discounts
			$p_price = self::__formatprc(
				self::__checkProductPrice(
					$product['PRICE'],
					$order['PRICE'] + $order['TAX_VALUE'],
					abs($order['DISCOUNT_VALUE'])
				)
			);

			// we clear the name of the product from special characters
			$p_name = preg_replace(
				'/[^\p{L}0-9 ]/iu',
				' ',
				html_entity_decode($product['NAME'])
			);
			$p_name = preg_replace('/ +/', ' ', trim($p_name));

			// form an array of goods for the check
			$r_products[$key_id] = new stdClass();
			$r_products[$key_id]->product_id = $product['PRODUCT_ID'];
			$r_products[$key_id]->product_name = $p_name;
			$r_products[$key_id]->product_item_price = $p_price;
			$r_products[$key_id]->product_quantity = $product['QUANTITY'];
			$r_products[$key_id]->product_tax = 0;
			$r_products[$key_id]->product_vendor_code = $productVendorCode;

			$key_id++;
		}

		// check parameters
		$items = '';

		// generate check data
		foreach ($r_products as $key_id => $product) {
			// display the product data in the check
			$items .= $items != '' ? ', ' : '';
			$items .= '{';
			$items .= '"calculationMethod":"' . $calc_method . '", ';
			$items .= '"paymentSubject":"PRODUCT", ';
			$items .= '"name":"' . $product->product_name . '", ';
			$items .= '"price":' . $product->product_item_price . ', ';
			$items .= '"quantity":' . $product->product_quantity . ', ';
			$items .=
				$vat_mode == 1
					? '"vat":"' . $vat . '", '
					: '"vat":"VAT_' . floor($product->product_tax) . '", ';
			$items .=
				'"total":' .
				self::__formatprc(
					$product->product_item_price * $product->product_quantity
				);
			$items .= '}';
		}

		// delivery
		if ($o_shipment > 0) {
			$shipment_tax_percent = 0;

			// display the data of the delivery method in the check
			$items .= $items != '' ? ', ' : '';
			$items .= '{';
			$items .= '"calculationMethod":"' . $calc_method . '", ';
			$items .= '"paymentSubject":"SERVICE", ';
			$items .= '"name":"' . GetMessage('BIFIT_DELIVERY') . '", ';
			$items .= '"price":' . self::__formatprc($o_shipment) . ', ';
			$items .= '"quantity":1, ';
			$items .=
				$vat_mode == 1
					? '"vat":"' . $vat . '", '
					: '"vat":"VAT_' . floor($shipment_tax_percent) . '", ';
			$items .= '"total":' . self::__formatprc($o_shipment);
			$items .= '}';
		}

		// form a check
		$receipt = '{';
		$receipt .=
			'"type":"' .
			$receipt_type .
			'", "taxSystem":"' .
			$tax_system .
			'", ';
		$receipt .= '"cashier":{';
		$receipt .= '"name":"' . $cashier_name . '", ';
		$receipt .= '"inn":"' . self::__checkInn($cashier_inn, 1) . '"';
		$receipt .= '}, ';
		$receipt .= '"client":{';
		$receipt .= $fio
			? '"name":"' .
				trim($user['LAST_NAME'] . ' ' . $user['NAME']) .
				'", '
			: '';
		$receipt .= '"address":"' . $address . '"';
		$receipt .= '}, ';
		$receipt .= '"paymentAddress":"' . $payment_address . '", ';
		$receipt .= '"items":[' . $items . '], ';
		$receipt .= '"total":' . self::__formatprc($order['PRICE']) . ', ';
		$receipt .=
			'"payments":{"' .
			$payment_type .
			'": ' .
			self::__formatprc($order['PRICE']) .
			'}';
		$receipt .= '}';

		// 3. SENDING A CHECK TO FISCAL PROCESSING
		//    transfer of the content of the check to fiscal processing for further fiscalization

		// address for POST request
		$url =
			'https://fp' .
			$test .
			'.bifit.com/processing-api/protected/documents/registration/receipts';

		// check data
		$data = $receipt;

		// Idempotency-Key
		$o_number = $order_number;
		$o_summ = $order['PRICE'];
		$date_time = $order['DATE_UPDATE']
			? $order['DATE_UPDATE']
			: date('Y-m-d H:i:s');
		$idemp_key = hash('sha256', "$o_number/$o_summ/$date_time");

		// headers
		$headers = array(
			'Content-Type: application/json',
			'Authorization: Bearer ' . $bifit->access_token,
			'Idempotency-Key: ' . $idemp_key
		);

		// get the ID of the document being processed (id)
		$receipt_id = json_decode(self::__loadData($url, $headers, $data));

		// 4. RECEIVING DOCUMENT BY ID
		//    obtaining a document with fiscal characteristics by its number

		// options
		$doc = '';

		if (!is_object($receipt_id)) {
			// link for GET request
			$url =
				'https://fp' .
				$test .
				'.bifit.com/processing-api/protected/documents/' .
				$receipt_id;

			// headers
			$headers = array('Authorization: Bearer ' . $bifit->access_token);

			// we get the document
			$doc = json_decode(self::__loadData($url, $headers, false));
		}

		// LINK TO THE CHECK

		// we form a link to the check
		$receipt_link = !is_object($receipt_id)
			? 'https://fp' .
				$test .
				'.bifit.com/processing-api/receipts/' .
				$idemp_key
			: 'XXX';

		// OPERATION HISTORY

		// object parameters
		$object = new stdClass();
		/// order id
		$object->order_id = $order_id;
		/// order number
		$object->order_number = $order_number;
		/// order date
		$object->order_date = $order['DATE_INSERT'];
		/// date of operation
		$object->action_date = date('Y-m-d H:i:s');
		/// link to check
		$object->receipt_link = $receipt_link;
		/// id of the received document
		$object->receipt_id = !is_object($receipt_id) ? $receipt_id : '0';
		/// type of document being held
		$object->receipt_type = $receipt_type;
		/// operation log (to be filled in later)
		$object->log = '';

		// save the operation to history
		self::__saveReport($test, $bifit, $receipt, $receipt_id, $doc, $object);
	}

	function __getLkData($order, $i)
	{
		// module id
		$moduleId = self::MODULE_ID;
		// tax system
		$tax_system = COption::GetOptionString(
			$moduleId,
			'tax_system_' . $i,
			'SIMPLIFIED_WITH_EXPENSE'
		);
		// method of calculation
		$calc_method = COption::GetOptionString(
			$moduleId,
			'calculation_method',
			'FULL_PAY'
		);
		// uniform VAT
		$vat_mode = COption::GetOptionString($moduleId, 'vat_mode', '1');
		// VAT
		$vat = COption::GetOptionString($moduleId, 'vat', 'WITHOUT_VAT');

		// buyer
		/// get the buyer's data
		$user = CUSer::GetByID($order['USER_ID'])->Fetch();
		/// buyer's phone
		$user_phone = 0;
		/// buyer's email
		$user_email = isset($user['EMAIL']) ? $user['EMAIL'] : 0;
		/// buyer's address
		$user_addr_arr = [];

		// order data
		/// order Id
		$order_id = (int) $order['ID'];

		$dbOrderProps = CSaleOrderPropsValue::GetList(
			["SORT" => "ASC"],
			["ORDER_ID" => $order_id]
		);

		$codes = ['LOCATION', 'ZIP', 'CITY', 'ADDRESS'];
		$more_codes = COption::GetOptionString($moduleId, 'more_codes', '');
		$more_codes =
			$more_codes != ''
				? explode(',', str_replace(' ', '', trim($more_codes)))
				: '';
		$codes = count($more_codes)
			? array_unique(array_merge($codes, $more_codes))
			: $codes;

		while ($arOrderProps = $dbOrderProps->Fetch()) {
			if (
				$arOrderProps['CODE'] == "EMAIL" &&
				$arOrderProps['VALUE'] != ''
			) {
				$user_email = $arOrderProps['VALUE'];
			}
			if (
				$arOrderProps['CODE'] == "PHONE" &&
				$arOrderProps['VALUE'] != ''
			) {
				$user_phone = $arOrderProps['VALUE'];
			}
			if (
				in_array($arOrderProps['CODE'], $codes) &&
				$arOrderProps['VALUE'] != ''
			) {
				if ($arOrderProps['CODE'] == 'LOCATION') {
					$res = \Bitrix\Sale\Location\LocationTable::getList(array(
						'filter' => array(
							'=CITY_ID' => $arOrderProps['VALUE'],
							'=PARENTS.NAME.LANGUAGE_ID' => LANGUAGE_ID,
							'=PARENTS.TYPE.NAME.LANGUAGE_ID' => LANGUAGE_ID
						),
						'select' => array(
							'I_ID' => 'PARENTS.ID',
							'I_NAME_RU' => 'PARENTS.NAME.NAME',
							'I_TYPE_CODE' => 'PARENTS.TYPE.CODE',
							'I_TYPE_NAME_RU' => 'PARENTS.TYPE.NAME.NAME'
						),
						'order' => array(
							'PARENTS.DEPTH_LEVEL' => 'asc'
						)
					));
					while ($item = $res->fetch()) {
						if (!in_array($item['I_NAME_RU'], $user_addr_arr)) {
							$user_addr_arr[] = $item['I_NAME_RU'];
						}
					}
				} else {
					$user_addr_arr[] = $arOrderProps['VALUE'];
				}
			}
		}

		$user_address = count($user_addr_arr)
			? implode(', ', $user_addr_arr) . '.'
			: '';
		$address_type = COption::GetOptionString(
			$moduleId,
			'address_type',
			'email'
		);

		if ($address_type == 'phone' && $user_phone) {
			$address = self::__checkPhone($user_phone);
		} elseif ($address_type == 'phone' && !$user_phone && $user_email) {
			$address = self::__checkEmail($user_email);
		} elseif ($address_type == 'email' && $user_email) {
			$address = self::__checkEmail($user_email);
		} elseif ($address_type == 'email' && !$user_email && $user_phone) {
			$address = self::__checkPhone($user_phone);
		} else {
			$address =
				$address_type == 'phone' && $user
					? self::__checkPhone($user_phone)
					: self::__checkEmail($user_email);
		}

		// products params
		$r_products = array();
		$dbBasket = CSaleBasket::GetList(
			["ID" => "ASC"],
			["ORDER_ID" => $order_id],
			false,
			false
		);
		$key_id = 0;

		while ($product = $dbBasket->GetNext()) {
			$res = CIBlockElement::GetByID($product['PRODUCT_ID']);
			if ($ar_res = $res->GetNext()) {
				$iblockid = $ar_res['IBLOCK_ID'];
			}
			$db_props = CIBlockElement::GetProperty(
				$iblockid,
				$product['PRODUCT_ID'],
				["SORT" => "ASC"],
				["CODE" => "ART_CODE"]
			);
			if ($ar_props = $db_props->Fetch()) {
				$productVendorCode = IntVal($ar_props["VALUE"]);
			}
			$p_price = self::__formatprc(
				self::__checkProductPrice(
					$product['PRICE'],
					$order['PRICE'] + $order['TAX_VALUE'],
					abs($order['DISCOUNT_VALUE'])
				)
			);
			$p_name = preg_replace(
				'/[^\p{L}0-9 ]/iu',
				' ',
				html_entity_decode($product['NAME'])
			);
			$p_name = preg_replace('/ +/', ' ', trim($p_name));
			$r_products[$key_id] = new stdClass();
			$r_products[$key_id]->product_id = $product['PRODUCT_ID'];
			$r_products[$key_id]->product_name = $p_name;
			$r_products[$key_id]->product_item_price = $p_price;
			$r_products[$key_id]->product_quantity = $product['QUANTITY'];
			$r_products[$key_id]->product_tax = 0;
			$r_products[$key_id]->product_vendor_code = $productVendorCode;
			$key_id++;
		}

		// lk data
		$lk = new stdClass();
		$lk->order_id = $order['ID'];
		$lk->order_number = $order['ACCOUNT_NUMBER'];
		$lk->order_price = $order['PRICE'];
		$lk->order_delivery_id = $order['DELIVERY_ID'];
		$lk->order_price_delivery = $order['PRICE_DELIVERY'];
		$lk->order_status_id = $order['STATUS_ID'];
		$lk->order_comments = trim(
			preg_replace('/\s\s+/', ' ', $order['USER_DESCRIPTION'])
		);
		$lk->order_payed = $order['PAYED'];
		$lk->order_date_payed = $order['DATE_PAYED'];
		$lk->user_name = $user['NAME'];
		$lk->user_last_name = $user['LAST_NAME'];
		$lk->user_email = self::__checkEmail($user_email);
		$lk->user_phone = self::__checkPhone($user_phone);
		$lk->user_address = $user_address;
		$lk->address = $address;
		$lk->r_products = $r_products;
		$lk->tax_system = $tax_system;
		$lk->calc_method = $calc_method;
		$lk->vat_mode = $vat_mode;
		$lk->vat = $vat;

		return $lk;
	}

	public static function handleSaleOrderStatus($id, $val)
	{
		if (gettype($id) == 'object' || !CModule::IncludeModule('sale')) {
			return;
		}

		$order = CSaleOrder::GetByID($id);
		$moduleId = self::MODULE_ID;

		// order data
		/// order status
		$status_id = $order['STATUS_ID'];
		/// payment method
		$payment_id = $order['PAY_SYSTEM_ID'];
		/// payment State
		$payed = $order['PAYED'];

		// plugin parameters
		/// selected operating mode
		$test = COption::GetOptionString($moduleId, 'test') ? '-test' : '';
		/// connection token
		$token =
			$test != ''
				? 'P5cKbUUD9uSSrSlGdzspLblvBnD0GzTAE0cLmAPSEMxJ79DtLE'
				: COption::GetOptionString($moduleId, 'token');

		// no action duplicates
		$no_duplicates = COption::GetOptionString(
			$moduleId,
			'no_duplicates',
			0
		);
		if ($no_duplicates) {
			$session = \Bitrix\Main\Application::getInstance()->getSession();
		}

		// iterating over the document type tabs
		for ($i = 1; $i <= 5; $i++) {
			// will find out whether it is necessary to use this type of document
			$use_type = COption::GetOptionString($moduleId, "use_type_{$i}")
				? 1
				: null;
			// if need to use
			if ($use_type) {
				// get parameters
				/// order paid (yes / no / disregard) (2/1/0)
				$order_paid = COption::GetOptionString(
					$moduleId,
					"order_paid_{$i}"
				);
				/// selected payment method (s)
				$pm_ids = json_decode(
					COption::GetOptionString($moduleId, "pm_ids_{$i}")
				);
				/// selected order status (s)
				$pm_statuses = json_decode(
					COption::GetOptionString($moduleId, "pm_statuses_{$i}")
				);
				/// use refused
				$use_refused = COption::GetOptionString(
					$moduleId,
					'lk_use_refused'
				)
					? 1
					: null;
				/// connection tab
				if ($i == 5 && $use_refused) {
					//// selected refused status (s)
					$lk_refused_statuses = json_decode(
						COption::GetOptionString(
							$moduleId,
							"lk_refused_statuses"
						)
					);
					/// if in array merge status arrays
					if (in_array($status_id, $lk_refused_statuses)) {
						$pm_statuses = array_merge(
							$pm_statuses,
							$lk_refused_statuses
						);
					}
				}

				// If the payment method and order status match
				if (
					isset($pm_ids) &&
					isset($pm_statuses) &&
					in_array($payment_id, $pm_ids) &&
					in_array($status_id, $pm_statuses) &&
					$token != ''
				) {
					// if "order paid" is selected,
					// and it is NOT paid, then we check the following tab
					if ($order_paid == '2' && $payed != 'Y') {
						continue;
					}

					// if "order not paid" is selected,
					// and it is paid, then we check the next tab
					if ($order_paid == '1' && $payed != 'N') {
						continue;
					}

					// check for duplicates
					if ($no_duplicates) {
						$act_name = 'bifit_act_' . $i . '_' . $id;
						if ($session->has($act_name)) {
							if (
								$session->get($act_name) >
								strtotime(date("Y-m-d H:i:s"))
							) {
								continue;
							}
						}
					}

					// main processes
					if ($i == 5) {
						// get lk-data
						$lk = self::__getLkData($order, $i);
						// update BIFIT-account data
						self::__saveOrder($lk);
					} else {
						// start BIFIT fiscal-process
						self::__mainProcess($order, $test, $token, $i);
					}

					// save to session and check old values
					if ($no_duplicates) {
						if (!$session->has('bifit_acts')) {
							$session['bifit_acts'] = [];
						} else {
							$arr = $session->get('bifit_acts');
							if (is_array($arr) && count($arr)) {
								foreach ($arr as $k => $v) {
									$a_n = 'bifit_act_' . $v;
									if ($session->has($a_n)) {
										if (
											strtotime(date("Y-m-d H:i:s")) >
											$session->get($a_n)
										) {
											$session->remove($a_n);
											unset($session['bifit_acts'][$k]);
										}
									}
								}
							}
						}
						$session['bifit_acts'][] = $i . '_' . $id;
						$session->set(
							$act_name,
							strtotime(date("Y-m-d H:i:s") . "+5 seconds")
						);
					}
				}
			}
		}
	}

	public static function handleSaleOrderPay($id, $val)
	{
		if (gettype($id) == 'object' || !CModule::IncludeModule('sale')) {
			return;
		}

		$moduleId = self::MODULE_ID;
		$order = CSaleOrder::GetByID($id);

		// order data
		/// order status
		$status_id = $order['STATUS_ID'];
		/// payment method
		$payment_id = $order['PAY_SYSTEM_ID'];
		/// payment State
		$payed = $order['PAYED'];

		$use_type = COption::GetOptionString($moduleId, "use_type_5")
			? 1
			: null;

		// if need to use
		if ($use_type && $payed == 'Y') {
			// get parameters
			/// order paid (yes / no / disregard) (2/1/0)
			$order_paid = COption::GetOptionString($moduleId, "order_paid_5");
			/// selected payment method (s)
			$pm_ids = json_decode(
				COption::GetOptionString($moduleId, "pm_ids_5")
			);
			/// selected order status (s)
			$pm_statuses = json_decode(
				COption::GetOptionString($moduleId, "pm_statuses_5")
			);

			// If the payment method and order status match
			if (
				isset($pm_ids) &&
				isset($pm_statuses) &&
				in_array($payment_id, $pm_ids) &&
				in_array($status_id, $pm_statuses)
			) {
				// if "order paid" is selected,
				// and it is NOT paid, then we check the following tab
				if ($order_paid == '2' && $payed != 'Y') {
					return;
				}

				// if "order not paid" is selected,
				// and it is paid, then we check the next tab
				if ($order_paid == '1' && $payed != 'N') {
					return;
				}

				// get lk-data
				$lk = self::__getLkData($order, 5);
				// update BIFIT-account data
				self::__saveOrder($lk);
			}
		}
	}
}
